@@ -16,6 +16,7 @@ from collections.abc import (
     Set as AbstractSet,
     KeysView,
 )
+from collections import ChainMap
 from enum import IntEnum
 from importlib import import_module
 from itertools import chain
@@ -44,7 +45,15 @@ import inflection
 from jinja2 import Environment, PackageLoader
 
 from .about import __version__
-from .typedef import parse_typedef, ismetasubclass, is_typing_definition, find_class_in_definition
+from .typedef import (
+    parse_typedef,
+    ismetasubclass,
+    is_typing_definition,
+    find_class_in_definition,
+    get_origin,
+    get_args,
+    Annotated,
+)
 from .typing import T, CellType, CoerceMapping, NoneType
 from .types import FrozenMapping, ReadOnly, AttrsDict, ClassOrInstanceFuncsDescriptor
 from .utils import flatten_fields
@@ -55,7 +64,9 @@ from .exceptions import (
     InvalidPostCoerceAttributeNames,
     CoerceMappingValueError,
     ClassCreationFailed,
+    RangeError,
 )
+from .constants import NoPickle, NoJSON, NoIterable, Range, NoHistory
 
 __version__  # Silence unused import warning.
 
@@ -92,6 +103,8 @@ def public_class(
         if key not in cls._slots:
             raise ValueError(f"{key!r} is not a field on {cls.__name__}!")
         next_cls = cls._slots[key]
+        if get_origin(next_cls) is Annotated:
+            next_cls, *_ = get_args(next_cls)
         if key in cls._nested_atomic_collection_keys:
             atomic_classes = cls._nested_atomic_collection_keys[key]
             if len(atomic_classes) > 1:
@@ -117,7 +130,7 @@ def public_class(
             else:
                 next_cls, = atomic_classes
         return public_class(next_cls, *rest, preserve_subtraction=preserve_subtraction)
-    cls = getattr(cls, "_parent", cls)
+    cls = cls.__public_class__()
     if preserve_subtraction and any((cls._skipped_fields, cls._modified_fields)):
         return cls
     if any((cls._skipped_fields, cls._modified_fields)):
@@ -129,6 +142,37 @@ def public_class(
     return cls
 
 
+def clear(instance: T, fields: Optional[Iterable[str]] = None) -> T:
+    """
+    Clear all fields on an instruct class instance.
+    """
+    if isinstance(instance, type):
+        raise TypeError("Can only call on an Atomic-metaclassed instance! You passed in a type!")
+    if not isinstance(instance.__class__, Atomic):
+        raise TypeError("Can only call on an Atomic-metaclassed type!")
+    if fields:
+        unrecognized_keys = frozenset(fields) - frozenset(keys(instance))
+        if unrecognized_keys:
+            instance._handle_init_errors([], [], unrecognized_keys)
+    instance._clear(fields=fields)
+    return instance
+
+
+def reset_to_defaults(instance: T, *, call_init_again: bool = False) -> T:
+    """
+    Clears an object and reinitializes to the values specified by
+    ``_set_defaults``.
+
+    call_init_again: call the instances `__init__` again (useful if you did
+    default initialization inside `__init__`).
+    """
+    clear(instance)
+    instance._set_defaults()
+    if call_init_again:
+        instance.__init__()
+    return instance
+
+
 def keys(
     instance_or_cls: Union[Type[T], T], *property_path: str, all: bool = False
 ) -> Union[AtomicKeysView, KeysView, Mapping[Type, KeysView]]:
@@ -378,14 +422,6 @@ def make_set_get_states(fields):
     return code_template
 
 
-def make_fast_new(fields, defaults_var_template):
-    defaults_var_template = env.from_string(defaults_var_template).render(fields=fields)
-    code_template = env.get_template("fast_new.jinja").render(
-        fields=fields, defaults_var_template=defaults_var_template
-    )
-    return code_template
-
-
 def make_defaults(fields, defaults_var_template):
     defaults_var_template = env.from_string(defaults_var_template).render(fields=fields)
     code = env.from_string(
@@ -393,7 +429,7 @@ def make_defaults(fields, defaults_var_template):
 def _set_defaults(self):
     result = self
     {{item|indent(4)}}
-    super(self.__class__, self)._set_defaults()
+    return super()._set_defaults()
     """
     ).render(item=defaults_var_template)
     return code
@@ -1034,6 +1070,15 @@ class Atomic(type):
         # i.e. key -> List[Union[AtomicDerived, bool]] means key can hold an Atomic derived type.
         _nested_atomic_collection_keys: Mapping[str, Tuple[Atomic, ...]]
 
+    def __public_class__(self):
+        """
+        Most of the time, we want to return the support class (the public facing one).
+
+        However, there are cases where a subclass changes the response of certain functions
+        and it is not desirable to return it for other code.
+        """
+        return getattr(self, "_parent", self)
+
     @classmethod
     def register_mixin(cls, name, klass):
         cls.MIXINS[name] = klass
@@ -1081,6 +1126,7 @@ class Atomic(type):
         effective_skipped_fields: FrozenMapping = skip_fields | currently_skipped_fields
         if not effective_skipped_fields:
             return self
+
         try:
             value = root_class.SKIPPED_FIELDS[(cls.__qualname__, effective_skipped_fields)]
         except KeyError:
@@ -1157,7 +1203,7 @@ class Atomic(type):
         if not changes:
             return self
         value = type(f"{cls.__name__}{changes}", (cls,), attrs, skip_fields=skip_entire_keys)
-        root_class.SKIPPED_FIELDS[(cls.__qualname__, value._skipped_fields)] = value
+        root_class.SKIPPED_FIELDS[(cls.__qualname__, effective_skipped_fields)] = value
         return value
 
     def __new__(
@@ -1187,26 +1233,38 @@ class Atomic(type):
         if include_fields and skip_fields:
             raise TypeError("Cannot specify both include_fields and skip_fields!")
         data_class_attrs = {}
+        base_class_functions = []
+        # Move overrides to the data class,
+        # so we call them first, then the codegen pieces.
+        # Suitable for a single level override.
+        # _set_defaults, __eq__ is special as it calls up the inheritance tree
+        # Others do not.
         for key in (
             "__iter__",
             "__getstate__",
             "__setstate__",
             "__eq__",
-            "__neq__",
-            "_set_defaults",
-            "clear",
             "__getitem__",
             "__setitem__",
         ):
             if key in attrs:
+                if hasattr(attrs[key], "_instruct_base_cls"):
+                    base_class_functions.append(key)
+                    continue
                 data_class_attrs[key] = attrs.pop(key)
-
+        base_class_functions = tuple(base_class_functions)
         support_cls_attrs = attrs
         del attrs
 
         if "__slots__" not in support_cls_attrs and "__annotations__" in support_cls_attrs:
             module = import_module(support_cls_attrs["__module__"])
-            hints = get_type_hints(SimpleNamespace(**support_cls_attrs), module.__dict__, globals())
+            hints = get_type_hints(
+                SimpleNamespace(**support_cls_attrs),
+                support_cls_attrs,
+                # First look in the module, then failsafe to the typing to support
+                # unimported 'Optional', et al
+                ChainMap(module.__dict__, typing.__dict__),
+            )
             support_cls_attrs["__slots__"] = hints
 
         if "__slots__" not in support_cls_attrs:
@@ -1279,8 +1337,8 @@ class Atomic(type):
             setter_templates.append(support_cls_attrs["__setter_template__"])
         if "__getter_template__" in support_cls_attrs:
             getter_templates.append(support_cls_attrs["__getter_template__"])
-        if "__defaults_init__" in support_cls_attrs:
-            defaults_templates.append(support_cls_attrs["__defaults_init__"])
+        if "__defaults__init__template__" in support_cls_attrs:
+            defaults_templates.append(support_cls_attrs["__defaults__init__template__"])
 
         # collection of all the known public properties for this class and it's parents:
         properties = [name for name, val in support_cls_attrs.items() if isinstance(val, property)]
@@ -1289,6 +1347,7 @@ class Atomic(type):
         if "__extra_slots__" in support_cls_attrs:
             pending_extra_slots.extend(support_cls_attrs["__extra_slots__"])
         # Base class inherited items:
+        annotated_metadata = {}
         inherited_listeners = {}
         for cls in bases:
             skipped_properties = ()
@@ -1320,6 +1379,8 @@ class Atomic(type):
             if ismetasubclass(cls, Atomic):
                 # Only Atomic Descendants will merge in the helpers of
                 # _columns: Dict[str, Type]
+                if cls._annotated_metadata:
+                    annotated_metadata.update(cls._annotated_metadata)
                 if cls._column_types:
                     column_types.update(cls._column_types)
                 if cls._nested_atomic_collection_keys:
@@ -1343,8 +1404,8 @@ class Atomic(type):
                     getter_templates.append(cls.__getter_template__)
                 if hasattr(cls, "__setter_template__"):
                     setter_templates.append(cls.__setter_template__)
-                if hasattr(cls, "__defaults_init__"):
-                    defaults_templates.append(cls.__defaults_init__)
+                if hasattr(cls, "__defaults__init__template__"):
+                    defaults_templates.append(cls.__defaults__init__template__)
             # Collect all publicly accessible properties:
             for key in dir(cls):
                 value = getattr(cls, key)
@@ -1384,6 +1445,8 @@ class Atomic(type):
                 no_op_skip_keys.append(key)
                 del combined_columns[key]
                 del combined_slots[key]
+                if key in annotated_metadata:
+                    del annotated_metadata[key]
 
             for key in tuple(current_class_columns.keys()):
                 if key not in skip_fields:
@@ -1398,6 +1461,8 @@ class Atomic(type):
                 no_op_skip_keys.append(key)
                 del combined_columns[key]
                 del combined_slots[key]
+                if key in annotated_metadata:
+                    del annotated_metadata[key]
 
             for key in tuple(current_class_columns.keys()):
                 if key in include_fields:
@@ -1416,10 +1481,15 @@ class Atomic(type):
         )
 
         if combined_columns:
+            try:
+                defaults_var_template = defaults_templates[0]
+            except IndexError:
+                raise MissingGetterSetterTemplateError(
+                    "You must define __defaults__init__template__!"
+                )
             try:
                 setter_var_template = setter_templates[0]
                 getter_var_template = getter_templates[0]
-                defaults_var_template = defaults_templates[0]
             except IndexError:
                 raise MissingGetterSetterTemplateError(
                     "You must define both __getter_template__ and __setter_template__"
@@ -1470,6 +1540,8 @@ class Atomic(type):
                         f"Disabling derived for {key} on {class_name}, failsafe to __coerce__[{coerce_types}]"
                     )
                 derived_class = None
+            if get_origin(value) is Annotated:
+                annotated_metadata[key] = value.__metadata__[:]
             new_property, isinstance_compatible_types = create_proxy_property(
                 env,
                 class_name,
@@ -1501,7 +1573,18 @@ class Atomic(type):
 
         ns_globals = {"NoneType": NoneType, "Flags": Flags, "typing": typing}
         ns_globals[class_name] = ReadOnly(None)
+
         if combined_columns:
+            exec(
+                compile(
+                    make_fast_dumps(combined_columns, class_name), "<make_fast_dumps>", mode="exec"
+                ),
+                ns_globals,
+                ns_globals,
+            )
+            class_cell_fixups.append(("_asdict", cast(FunctionType, ns_globals["_asdict"])))
+            class_cell_fixups.append(("_astuple", cast(FunctionType, ns_globals["_astuple"])))
+            class_cell_fixups.append(("_aslist", cast(FunctionType, ns_globals["_aslist"])))
             exec(
                 compile(make_fast_eq(combined_columns), "<make_fast_eq>", mode="exec"),
                 ns_globals,
@@ -1516,16 +1599,7 @@ class Atomic(type):
                 ns_globals,
                 ns_globals,
             )
-            exec(
-                compile(
-                    make_fast_dumps(combined_columns, class_name), "<make_fast_dumps>", mode="exec"
-                ),
-                ns_globals,
-                ns_globals,
-            )
-            class_cell_fixups.append(("_asdict", cast(FunctionType, ns_globals["_asdict"])))
-            class_cell_fixups.append(("_astuple", cast(FunctionType, ns_globals["_astuple"])))
-            class_cell_fixups.append(("_aslist", cast(FunctionType, ns_globals["_aslist"])))
+            class_cell_fixups.append(("clear", cast(FunctionType, ns_globals["_clear"])))
             exec(
                 compile(
                     make_fast_getset_item(
@@ -1541,15 +1615,24 @@ class Atomic(type):
                 ns_globals,
                 ns_globals,
             )
+            iter_fields = []
+            for field in combined_columns:
+                if field in annotated_metadata and NoIterable in annotated_metadata[field]:
+                    continue
+                iter_fields.append(field)
             exec(
-                compile(make_fast_iter(combined_columns), "<make_fast_iter>", mode="exec"),
+                compile(make_fast_iter(iter_fields), "<make_fast_iter>", mode="exec"),
                 ns_globals,
                 ns_globals,
             )
+            del iter_fields
+            pickle_fields = []
+            for field in combined_columns:
+                if field in annotated_metadata and NoPickle in annotated_metadata[field]:
+                    continue
+                pickle_fields.append(field)
             exec(
-                compile(
-                    make_set_get_states(combined_columns), "<make_set_get_states>", mode="exec"
-                ),
+                compile(make_set_get_states(pickle_fields), "<make_set_get_states>", mode="exec"),
                 ns_globals,
                 ns_globals,
             )
@@ -1562,26 +1645,15 @@ class Atomic(type):
                 ns_globals,
                 ns_globals,
             )
-        if "__new__" not in support_cls_attrs and combined_columns:
-            exec(
-                compile(
-                    make_fast_new(current_class_fields, defaults_var_template),
-                    "<make_fast_new>",
-                    mode="exec",
-                ),
-                ns_globals,
-                ns_globals,
+            class_cell_fixups.append(
+                ("_set_defaults", cast(FunctionType, ns_globals["_set_defaults"]))
             )
-            support_cls_attrs["__new__"] = new_new = ns_globals.pop("__new__")
-            class_cell_fixups.append(("__new__", cast(FunctionType, new_new)))
         for key in (
             "__iter__",
             "__getstate__",
             "__setstate__",
             "__eq__",
-            "__neq__",
-            "_set_defaults",
-            "clear",
+            "_clear",
             "__getitem__",
             "__setitem__",
             "_asdict",
@@ -1592,8 +1664,12 @@ class Atomic(type):
             # Any overrides that *may* call them will be assigned
             # to the concrete class instead
             if key in ns_globals:
+                if key in base_class_functions:
+                    continue
                 logger.debug(f"Copying {key} into {class_name} attributes")
                 support_cls_attrs[key] = ns_globals.pop(key)
+        if "_set_defaults" in ns_globals:
+            data_class_attrs["_set_defaults"] = ns_globals.pop("_set_defaults")
 
         # Any keys subtracted must have no-nop setters in order to
         # allow for subtype relationship will behaving as if those keys are fundamentally
@@ -1616,6 +1692,7 @@ class Atomic(type):
         support_cls_attrs["_column_types"] = ReadOnly(FrozenMapping(column_types))
         support_cls_attrs["_all_coercions"] = ReadOnly(FrozenMapping(all_coercions))
         support_cls_attrs["_support_columns"] = tuple(support_columns)
+        support_cls_attrs["_annotated_metadata"] = ReadOnly(FrozenMapping(annotated_metadata))
         support_cls_attrs["_nested_atomic_collection_keys"] = FrozenMapping(
             nested_atomic_collections
         )
@@ -1696,9 +1773,16 @@ class Atomic(type):
         }
         for instance in instances:
             instance_type = type(instance)
+            annotated_metadata = instance_type._annotated_metadata
+            skip_fields = set()
+            for key in annotated_metadata:
+                if NoJSON in annotated_metadata[key]:
+                    skip_fields.add(key)
             result = {}
             special_binary_encoders = cached_class_binary_encoders[instance_type]
-            for key, value in instance:
+            for key, value in instance._asdict().items():
+                if key in skip_fields:
+                    continue
                 # Support nested daos
                 if hasattr(value, "to_json"):
                     value = value.to_json()
@@ -1749,7 +1833,7 @@ UNDEFINED = Undefined()
 
 
 class History(metaclass=Atomic):
-    __slots__ = ("_changes", "_changed_keys", "_changed_index")
+    __slots__ = ("_changes", "_changed_keys", "_changed_index", "_suppress_history")
     setter_wrapper = "history-setter-wrapper.jinja"
 
     if TYPE_CHECKING:
@@ -1757,8 +1841,17 @@ class History(metaclass=Atomic):
 
     def __init__(self, **kwargs):
         t_s = time.time()
-        self._changes = {key: [Delta("default", UNDEFINED, value, 0)] for key, value in self}
-        self._changed_keys = [(key, t_s) for key in self._columns]
+        self._suppress_history = frozenset(
+            field for field, metadata in self._annotated_metadata.items() if NoHistory in metadata
+        )
+        self._changes = {
+            key: [Delta("default", UNDEFINED, value, 0)]
+            for key, value in self._asdict().items()
+            if key not in self._suppress_history
+        }
+        self._changed_keys = [
+            (key, t_s) for key in self._columns if key not in self._suppress_history
+        ]
         self._changed_index = len(self._changed_keys)
 
         super().__init__(**kwargs)
@@ -1775,6 +1868,8 @@ class History(metaclass=Atomic):
             return
         if self._flags & Flags.DISABLE_HISTORY:
             return
+        if key in self._suppress_history:
+            return
         msg = "update"
         if self._flags & 2 == 2:
             msg = "initialized"
@@ -1964,6 +2059,15 @@ class JSONSerializable(metaclass=Atomic):
         return tuple(cls.from_json(item) for item in iterable)
 
 
+def mark(**kwargs):
+    def wrapper(func):
+        for key, value in kwargs.items():
+            setattr(func, f"_instruct_{key}", value)
+        return func
+
+    return wrapper
+
+
 Atomic.register_mixin("json", JSONSerializable)
 
 
@@ -1971,23 +2075,26 @@ class SimpleBase(metaclass=Atomic):
     __slots__ = ("_flags",)
     __setter_template__ = ReadOnly("self._{key}_ = val")
     __getter_template__ = ReadOnly("return self._{key}_")
-    __defaults_init__ = ReadOnly(DEFAULTS)
+    __defaults__init__template__ = ReadOnly(DEFAULTS)
 
+    @mark(base_cls=True)
     def __new__(cls, *args, **kwargs):
         # Get the edge class that has all the __slots__ defined
         cls = cls._data_class
         result = super().__new__(cls)
         result._flags = Flags.UNCONSTRUCTED
+        result._set_defaults()
         assert "__dict__" not in dir(
             result
         ), "Violation - there should never be __dict__ on a slotted class"
         return result
 
     def __len__(self):
-        return len(keys(self.__class__))
+        cls = public_class(self, preserve_subtraction=True)
+        return len(keys(cls))
 
     def __contains__(self, item):
-        cls = type(self)
+        cls = public_class(self, preserve_subtraction=True)
         if item in cls._skipped_fields:
             return False
         return item in cls._all_accessible_fields
@@ -1995,7 +2102,9 @@ class SimpleBase(metaclass=Atomic):
     def __reduce__(self):
         # Create an empty class then let __setstate__ in the autogen
         # code to handle passing raw values.
-        cls = public_class(type(self))
+
+        # Get the public, unmessed with class:
+        cls = public_class(self)
         skipped_fields = _dump_skipped_fields(type(self))
         return load_cls, (cls, (), {}, skipped_fields), self.__getstate__()
 
@@ -2048,6 +2157,7 @@ class SimpleBase(metaclass=Atomic):
             raise TypeError(
                 f"__init__() takes {len(class_keys)} positional arguments but {len(args)} were given"
             )
+        # Set by argument position
         for key, value in zip(class_keys, args):
             try:
                 setattr(self, key, value)
@@ -2055,6 +2165,7 @@ class SimpleBase(metaclass=Atomic):
                 errors.append(e)
                 errored_keys.append(key)
         class_keys = self._all_accessible_fields
+        # Set by keywords
         for key in class_keys & kwargs.keys():
             value = kwargs[key]
             try:
@@ -2068,9 +2179,37 @@ class SimpleBase(metaclass=Atomic):
         self._handle_init_errors(errors, errored_keys, unrecognized_keys)
         self._flags = Flags.INITIALIZED
 
-    def clear(self, fields=None):
+    @mark(base_cls=True)
+    def _clear(self, fields: Iterable[str] = None):
         pass
 
+    @mark(base_cls=True)
+    def _asdict(self) -> Dict[str, Any]:
+        return {}
+
+    @mark(base_cls=True)
+    def _astuple(self) -> Tuple[Any, ...]:
+        return ()
+
+    @mark(base_cls=True)
+    def _aslist(self) -> List[Any]:
+        return []
+
+    @mark(base_cls=True)
+    def _set_defaults(self):
+        # ARJ: Override to set defaults instead of inside the `__init__` function
+        # Note: Always call ``super()._set_defaults()`` FIRST as if you
+        # call it afterwards, the inheritance tree will zero initialize it first
+        return self
+
+    @mark(base_cls=True)
+    def __iter__(self):
+        """
+        Dummy iter.
+        """
+        if False:
+            yield
+
 
 class Base(SimpleBase, mapping=True, json=True):
     __slots__ = ()
@@ -2085,6 +2224,8 @@ __all__ = [
     "values",
     "items",
     "get",
+    "clear",
+    "reset_to_defaults",
     "asdict",
     "astuple",
     "aslist",
@@ -2109,4 +2250,10 @@ __all__ = [
     "JSONSerializable",
     # misc
     "UNDEFINED",
+    "NoJSON",
+    "NoPickle",
+    "NoIterable",
+    "NoHistory",
+    "Range",
+    "RangeError",
 ]  # noqa
